import xml.etree.ElementTree as eT
import random


class Hold:
    def __init__(self, centers, mesh_name):
        self.center_x = centers[0]
        self.center_y = centers[1]
        self.size_x = centers[0] * 2
        self.size_y = centers[1] * 2
        self.mesh = mesh_name


def insert_wall(file_name):
    hold_1 = Hold((0.07, 0.05), 'rock_hold_1')
    hold_2 = Hold((0.07, 0.07), 'rock_hold_2')
    hold_3 = Hold((0.07, 0.08), 'rock_hold_3')
    hold_4 = Hold((0.07, 0.08), 'rock_hold_4')
    hold_5 = Hold((0.07, 0.08), 'rock_hold_5')
    hold_6 = Hold((0.07, 0.08), 'rock_hold_6')
    holds = [hold_1, hold_2, hold_3, hold_4, hold_5, hold_6]

    center = 2
    size = center * 2

    tree = eT.ElementTree(file=file_name)
    world_body = tree.find('worldbody')

    # Add wall body
    body = eT.SubElement(world_body, 'body')
    body.attrib['name'] = 'rock_wall'

    # Add wall geom
    wall = eT.SubElement(body, 'geom')
    wall.attrib['name'] = 'wall'
    wall.attrib['pos'] = '0 -1 ' + str(center)
    wall.attrib['material'] = 'geom'
    wall.attrib['rgba'] = '.4 .9 .9 1'
    wall.attrib['size'] = str(center) + ' 0.1 ' + str(center)
    wall.attrib['type'] = 'box'

    assets = tree.find('asset')

    # Load stl files as mesh assets
    for ind in range(1, 7):
        mesh = eT.SubElement(assets, 'mesh')
        mesh.attrib['file'] = 'Rock_hold_' + str(ind) + '_meshlab.stl'
        mesh.attrib['name'] = 'rock_hold_' + str(ind)
        if ind == 2:
            mesh.attrib['scale'] = '0.0008 0.0008 0.0008'
        else:
            mesh.attrib['scale'] = '0.002 0.002 0.002'

    hold_count = 1
    # add_hold(0, 1, 'hold_' + str(hold_count), get_random_hold(holds), body, center)

    holds_per_tile = 4
    filled_heights = [0] * int(size * holds_per_tile)
    for ind in range(0, len(filled_heights)):
        filled_heights[ind] = random.uniform(0.2, 0.6)
    filled_count = 0
    while filled_count < len(filled_heights)-2:
        filled_count = 0
        shift = random.uniform(0, 0.1)
        for i in range(0, len(filled_heights)):
            if filled_heights[i] < size - 0.2:
                if random.randint(0, 1) == 1:
                    rand_hold = get_random_hold(holds)
                    add_hold(i / holds_per_tile + shift,
                             filled_heights[i], 'hold_' + str(hold_count), rand_hold, body, center)
                    hold_count += 1
                    filled_heights[i] += random.uniform(0.5, 1)  # rand_hold.size_y
                # filled_heights[i] += 1
            else:
                filled_count += 1

    indent(tree.getroot())
    return tree


def generate_random_wall():
    hold_1 = Hold((0.7, 0.5), 'rock_hold_1')
    hold_2 = Hold((0.7, 0.7), 'rock_hold_2')
    hold_3 = Hold((0.7, 0.8), 'rock_hold_3')
    hold_4 = Hold((0.7, 0.8), 'rock_hold_4')
    hold_5 = Hold((0.7, 0.8), 'rock_hold_5')
    hold_6 = Hold((0.7, 0.8), 'rock_hold_6')
    holds = [hold_1, hold_2, hold_3, hold_4, hold_5, hold_6]
    # create a wall of size wall_size X wall_size
    wall_size = 20
    wall_center = wall_size / 2

    root = eT.Element("mujoco")
    root.attrib['model'] = 'rockwall'
    tree = eT.ElementTree(root)

    world_body = eT.SubElement(root, 'worldbody')

    # Add floor geom
    floor = eT.SubElement(world_body, 'geom')
    floor.attrib['name'] = 'floor'
    floor.attrib['pos'] = '0 0 0'
    floor.attrib['size'] = '50 50 .125'
    floor.attrib['type'] = 'plane'
    floor.attrib['material'] = 'MatPlane'
    floor.attrib['condim'] = '3'

    # Add light
    light = eT.SubElement(world_body, 'light')
    light.attrib['mode'] = 'targetbodycom'
    light.attrib['target'] = 'rock_wall'
    light.attrib['directional'] = 'false'
    light.attrib['diffuse'] = '.8 .8 .8'
    light.attrib['specular'] = '0.3 0.3 0.3'
    light.attrib['pos'] = '0 0 4.0'
    light.attrib['dir'] = '0 0 -1'

    # Add camera
    camera = eT.SubElement(world_body, 'camera')
    camera.attrib['name'] = 'targeting'
    light.attrib['target'] = 'rock_wall'
    light.attrib['pos'] = '1 1 2'

    # Add main body
    body = eT.SubElement(world_body, 'body')
    body.attrib['name'] = 'rock_wall'

    # Add wall geom
    wall = eT.SubElement(body, 'geom')
    wall.attrib['name'] = 'wall'
    wall.attrib['pos'] = '0 0 ' + str(wall_center)
    wall.attrib['material'] = 'geom'
    wall.attrib['rgba'] = '.4 .9 .9 1'
    wall.attrib['size'] = str(wall_center) + ' 1 ' + str(wall_center)
    wall.attrib['type'] = 'box'

    assets = eT.SubElement(root, 'asset')

    # Add texture for floor
    floor_tex = eT.SubElement(assets, 'texture')
    floor_tex.attrib['name'] = 'texplane'
    floor_tex.attrib['type'] = '2d'
    floor_tex.attrib['builtin'] = 'checker'
    floor_tex.attrib['rgb1'] = '.2 .3 .4'
    floor_tex.attrib['rgb2'] = '.1 0.15 0.2'
    floor_tex.attrib['width'] = '512'
    floor_tex.attrib['height'] = '512'

    floor_mat = eT.SubElement(assets, 'material')
    floor_mat.attrib['name'] = 'MatPlane'
    floor_mat.attrib['reflectance'] = '0.3'
    floor_mat.attrib['texture'] = 'texplane'
    floor_mat.attrib['texrepeat'] = '1 1'
    floor_mat.attrib['texuniform'] = 'true'

    # Add texture for wall
    wall_tex = eT.SubElement(assets, 'texture')
    wall_tex.attrib['name'] = 'texgeom'
    wall_tex.attrib['type'] = 'cube'
    wall_tex.attrib['builtin'] = 'flat'
    wall_tex.attrib['rgb1'] = '0.8 0.6 0.4'
    wall_tex.attrib['rgb2'] = '0.8 0.6 0.4'
    wall_tex.attrib['width'] = '127'
    wall_tex.attrib['height'] = '1278'
    wall_tex.attrib['markrgb'] = '1 1 1'
    wall_tex.attrib['random'] = '0.01'

    wall_mat = eT.SubElement(assets, 'material')
    wall_mat.attrib['name'] = 'geom'
    wall_mat.attrib['texture'] = 'texgeom'
    wall_mat.attrib['texuniform'] = 'true'

    # Load stl files as mesh assets
    for ind in range(1, 7):
        mesh = eT.SubElement(assets, 'mesh')
        mesh.attrib['file'] = 'Rock_hold_' + str(ind) + '_meshlab.stl'
        mesh.attrib['name'] = 'rock_hold_' + str(ind)
        if ind == 2:
            mesh.attrib['scale'] = '0.008 0.008 0.008'
        else:
            mesh.attrib['scale'] = '0.02 0.02 0.02'

    filled_heights = [0] * int(wall_center)
    for ind in range(0, len(filled_heights)):
        filled_heights[ind] = 0.5 + random.random()
    filled_count = 0
    hold_count = 1
    while filled_count < 2:
        filled_count = 0
        shift = random.uniform(0, 0.6)
        for i in range(0, len(filled_heights)):
            if filled_heights[i] < wall_size - 2:
                if random.randint(0, 1) == 1:
                    rand_hold = get_random_hold(holds)
                    add_hold(i * 2 + shift, filled_heights[i], 'hold_' + str(hold_count), rand_hold, body, wall_center)
                    hold_count += 1
                    filled_heights[i] += rand_hold.size_y
                filled_heights[i] += 2
            else:
                filled_count += 1
    indent(tree.getroot())
    return tree


def add_hold(x, y, name, hold, body, wall_center_pos):
    new_hold = eT.SubElement(body, 'geom')
    new_hold.attrib['name'] = name
    new_hold.attrib['axisangle'] = '1 0 0 90'
    new_hold.attrib['mesh'] = hold.mesh
    new_hold.attrib['rgba'] = str(random.randint(0, 1)) + ' ' + str(random.randint(0, 1)) + ' ' + str(
        random.randint(0, 1)) + ' 1'
    new_hold.attrib['type'] = 'mesh'
    new_hold.attrib['pos'] = str(x - wall_center_pos + hold.center_x) + ' -1.1 ' + str(y + hold.center_y)


def get_random_hold(holds):
    return holds[random.randint(0, len(holds) - 1)]


def indent(elem, level=0):
    i = "\n\n" + level * "  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
        for elem in elem:
            indent(elem, level + 1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i


if __name__ == '__main__':
    t = generate_random_wall()
    indent(t.getroot())
    t.write('test.xml')
